# -*- coding: utf-8-*-
import sys
import logging
import socket
import functools
import pkgutil
if sys.version_info < (3, 3):
    from distutils.spawn import find_executable  # pylint: disable=E0611
else:
    from shutil import which as find_executable  # pylint: disable=E0611


def check_executable(executable):
    """
    Checks if an executable exists in $PATH.
    Arguments:
        executable -- the name of the executable (e.g. "echo")
    Returns:
        True or False
    """
    logger = logging.getLogger(__name__)
    logger.debug("Checking executable '%s'...", executable)
    executable_path = find_executable(executable)
    found = executable_path is not None
    if found:
        logger.debug("Executable '%s' found: '%s'", executable, executable_path)
    else:
        logger.debug("Executable '%s' not found", executable)
    return found


def check_network_connection(server, port):
    """
    Checks if jasper can connect a network server.
    Arguments:
        server -- (optional) the server to connect with (Default:
                  "www.google.com")
    Returns:
        True or False
    """
    logger = logging.getLogger(__name__)
    logger.debug("Checking network connection to server '%s'...", server)
    try:
        # see if we can resolve the host name -- tells us if there is
        # a DNS listening
        host = socket.gethostbyname(server)
        # connect to the host -- tells us if the host is actually
        # reachable
        sock = socket.create_connection((host, port), 2)
        sock.close()
    except Exception:  # pragma: no cover
        logger.debug("Network connection not working")
        return False
    logger.debug("Network connection working")
    return True


def check_python_import(package_or_module):
    """
    Checks if a python package or module is importable.
    Arguments:
        package_or_module -- the package or module name to check
    Returns:
        True or False
    """
    logger = logging.getLogger(__name__)
    logger.debug("Checking python import '%s'...", package_or_module)
    loader = pkgutil.get_loader(package_or_module)
    found = loader is not None
    if found:
        logger.debug("Python %s '%s' found",
                     "package" if loader.is_package(package_or_module)
                     else "module", package_or_module)
    else:  # pragma: no cover
        logger.debug("Python import '%s' not found", package_or_module)
    return found


def process_options(valid_options, _options, error):
    unknown_options = set(_options.keys()).difference(valid_options.keys())
    if unknown_options:
        raise error('Unknown options: %s' % ', '.join(unknown_options))

    options = dict((key, _options.get(key, val.get('default', None))) for key, val in valid_options.items())
    for option in options.keys():
        val = options[option]
        data = valid_options[option]
        typ = data['type']
        if typ not in ['int', 'float', 'str', 'enum', 'bool']:
            raise error('Bad type: %s for option %s' % (typ, option), ['int', 'float', 'str', 'enum', 'bool'])
        keys = data.keys()
        if typ == 'int':
            val = int(float(val))
        if typ == 'float':
            val = float(val)
        if typ in ['int', 'float']:
            if 'min' in keys:
                if val < data['min']:
                    raise error('Bad %s: %s' % (option, val), 'Min is %s' % data['min'])
            if 'max' in keys:
                if val > data['max']:
                    raise error('Bad %s: %s' % (option, val), 'Max is %s' % data['max'])
        if typ == 'enum':
            if val not in data['values']:
                raise error('Bad %s value: %s' % (option, val), data['values'])
        if typ == 'bool':
            val = True if str(val).lower() in ['y', '1', 'yes', 'true', 't'] else False
        options[option] = val
    return options



import os
import re
import tempfile
import pipes
from talkey.base import AbstractTTSEngine, subprocess, register
from talkey.utils import check_executable


@register
class PicoTTS(AbstractTTSEngine):
    """
    Uses the svox-pico-tts speech synthesizer.

    Requires ``pico2wave`` to be available.
    """

    SLUG = "pico"

    @classmethod
    def _get_init_options(cls):
        return {
            'pico2wave': {
                'description': 'pico2wave executable path',
                'type': 'str',
                'default': 'pico2wave'
            },
        }

    def _is_available(self):
        return check_executable(self.ioptions['pico2wave'])

    def _get_options(self):
        return {}

    def _get_languages(self):
        cmd = ['pico2wave', '-l', 'NULL', '-w', os.devnull]
        with tempfile.SpooledTemporaryFile() as f:
            subprocess.call(cmd, stderr=f)
            f.seek(0)
            output = f.read().decode('utf-8')
        pattern = re.compile(r'Unknown language: NULL\nValid languages:\n((?:[a-z]{2}-[A-Z]{2}\n)+)')
        matchobj = pattern.match(output)
        voices = matchobj.group(1).split()
        langs = {}
        for voice in voices:
            lang = voice[:2]
            langs.setdefault(lang, {'default': voice, 'voices': {}})
            langs[lang]['voices'][voice] = {}
        return langs

    def _say(self, phrase, language, voice, voiceinfo, options):
        with tempfile.NamedTemporaryFile(suffix='.wav', delete=False) as f:
            fname = f.name
        cmd = ['pico2wave', '-l', voice, '-w', fname, phrase]
        self._logger.debug('Executing %s', ' '.join([pipes.quote(arg) for arg in cmd]))
        subprocess.call(cmd)
        self.play(fname)
        os.remove(fname)



from .dummy import DummyTTS
from .festival import FestivalTTS
from .flite import FliteTTS
from .espeak import EspeakTTS
from .mary import MaryTTS
from .pico import PicoTTS
from .google import GoogleTTS

_ENGINE_MAP = {
    'dummy': DummyTTS,
    'flite': FliteTTS,
    'pico': PicoTTS,
    'festival': FestivalTTS,
    'espeak': EspeakTTS,
    'mary': MaryTTS,
    'google': GoogleTTS,
}

_ENGINE_ORDER = ['google', 'mary', 'espeak', 'festival', 'pico', 'flite', 'dummy']



import os
import tempfile

try:
    import gtts
except ImportError:  # pragma: no cover
    pass

from talkey.base import AbstractTTSEngine, register
from talkey.utils import check_network_connection, check_python_import


@register
class GoogleTTS(AbstractTTSEngine):
    """
    Uses the Google TTS online translator.

    Requires module ``gTTS`` to be available.
    """

    SLUG = "google"

    @classmethod
    def _get_init_options(cls):
        return {
            'enabled': {
                'description': 'Is enabled?',
                'type': 'bool',
                'default': False,
            },
        }

    def _is_available(self):
        return (
            check_python_import('gtts')
            and check_network_connection('translate.google.com', 80)
        )

    def _get_options(self):
        return {}

    def _get_languages(self):
        voices = gtts.gTTS.LANGUAGES.keys()
        langs = {}
        for voice in voices:
            lang = voice[:2]
            langs.setdefault(lang, {'default': voice, 'voices': {}})
            langs[lang]['voices'][voice] = {}
        return langs

    def _say(self, phrase, language, voice, voiceinfo, options):
        tts = gtts.gTTS(text=phrase, lang=voice)
        with tempfile.NamedTemporaryFile(suffix='.mp3', delete=False) as f:
            tmpfile = f.name
        tts.save(tmpfile)
        self.play(tmpfile, translate=True)
        os.remove(tmpfile)



import langid

from .base import TTSError
from .engines import _ENGINE_MAP, _ENGINE_ORDER


def enumerate_engines():
    '''
    Returns list of engine SLUGs in order of preference
    '''
    return _ENGINE_ORDER


def create_engine(engine, options=None, defaults=None):
    '''
    Creates an instance of an engine.
    There is a two-stage instantiation process with engines.

    1. ``options``:
        The keyword options to instantiate the engine class
    2. ``defaults``:
        The default configuration for the engine (options often depends on instantiated TTS engine)
    '''
    if engine not in _ENGINE_MAP.keys():
        raise TTSError('Unknown engine %s' % engine)

    options = options or {}
    defaults = defaults or {}
    einst = _ENGINE_MAP[engine](**options)
    einst.configure_default(**defaults)
    return einst


class Talkey(object):
    '''
    Manages engines and allows multi-lingual say()

    ``preferred_languages``
        A list of languages that are weighted in preference. This is a weighting to assist the detection of language by classify().
    ``preferred_factor``
        The weighting factor to prefer the ``preferred_languages`` list. Higher number skews towards preference.
    ``engine_preference``
        Specify preferred engines in order of preference.
    ``**config``
        Engine-specfic configuration, e.g.:

        .. code-block:: python

            # Key is the engine SLUG, in this case ``espeak``
            espeak={
                # Specify the engine options:
                'enabled': True,

                # Specify some default voice options
                'defaults': {
                        'words_per_minute': 150,
                        'variant': 'f4',
                },

                # Here you specify language-specific voice options
                # e.g. for english we prefer the mbrola en1 voice
                'languages': {
                    'en': {
                        'voice': 'english-mb-en1',
                        'words_per_minute': 130
                    },
                }
            }
    '''

    def __init__(self, preferred_languages=None, preferred_factor=80.0, engine_preference=None, **config):
        self.preferred_languages = preferred_languages or []
        self.preferred_factor = preferred_factor
        engine_preference = engine_preference or enumerate_engines()
        for ename in enumerate_engines():
            if ename not in engine_preference:
                engine_preference.append(ename)

        self.engines = []
        self.languages = set()

        for ename in engine_preference:
            try:
                options = config.get(ename, {}).get('options', {})
                defaults = config.get(ename, {}).get('defaults', {})
                eng = create_engine(ename, options=options, defaults=defaults)
                self.engines.append(eng)

                languages = config.get(ename, {}).get('languages', {})
                for lang, conf in languages.items():
                    eng.configure(language=lang, **conf)
            except TTSError:
                pass

        for eng in self.engines:
            self.languages.update(eng.languages.keys())

        langid.set_languages(self.languages)

        if not self.languages:
            raise TTSError('No supported languages')

    def classify(self, txt):
        '''
        Classifies text by language. Uses preferred_languages weighting.
        '''
        ranks = []
        for lang, score in langid.rank(txt):
            if lang in self.preferred_languages:
                score *= self.preferred_factor
            ranks.append((lang, score))
        ranks.sort(key=lambda x: x[1], reverse=True)
        return ranks[0][0]

    def get_engine_for_lang(self, lang):
        '''
        Determines the preferred engine/voice for a language.
        '''
        for eng in self.engines:
            if lang in eng.languages.keys():
                return eng
        raise TTSError('Could not match language')

    def say(self, txt, lang=None):
        '''
        Says the text.

        if ``lang`` is ``None``, then uses ``classify()`` to detect language.
        '''
        lang = lang or self.classify(txt)
        self.get_engine_for_lang(lang).say(txt, language=lang)



import os
import pipes
import logging
import tempfile
from abc import ABCMeta, abstractmethod

try:
    import subprocess32 as subprocess
except ImportError:
    import subprocess


from talkey.utils import process_options, check_executable

import langid
import contextlib
import audioread
import wave

# Get the list of identifiable languages
DETECTABLE_LANGS = sorted([a[0] for a in langid.rank('')])


def genrst(label, opt, txt, indent='    '):
    txt += '\n%s%s:\n\n' % (indent, label)
    for key in sorted(opt.keys()):
        val = opt[key]
        txt += indent + '``%s``\n' % key
        txt += indent + '    %s\n\n' % val.get('description', '%s option' % key)
        txt += indent + '    :type: %s\n' % val['type']
        txt += indent + '    :default: %s\n' % val['default']
        if 'min' in val.keys():
            txt += indent + '    :min: %s\n' % val['min']
        if 'max' in val.keys():
            txt += indent + '    :max: %s\n' % val['max']
        if 'values' in val.keys():
            txt += indent + '    :values: %s\n' % ', '.join(val['values'])
    return txt


def register(cls):
    cls.__doc__ = genrst('Initialization options', cls.get_init_options(), cls.__doc__)
    return cls


class TTSError(Exception):
    '''
    The exception that Talkey will throw if any error occurs.
    '''

    def __init__(self, error, valid_set=None):  # pylint: disable=W0231
        self.error = error
        self.valid_set = valid_set

    def __str__(self):
        if self.valid_set:
            return '%s\nValid set: %s' % (self.error, self.valid_set)
        else:
            return self.error


class AbstractTTSEngine(object):
    """
    Generic parent class for all speakers
    """
    __metaclass__ = ABCMeta
    SLUG = None
    'The SLUG is used to identify the engine as text'

    # Define these in your engine
    @classmethod
    @abstractmethod
    def _get_init_options(cls):
        'AbstractMethod: Returns dict of engine options'
        pass  # pragma: no cover

    @abstractmethod
    def _is_available(self):
        'AbstractMethod: Boolean on if engine is available'
        pass  # pragma: no cover

    @abstractmethod
    def _get_options(self):
        'AbstractMethod: Returns dict of voice options'
        pass  # pragma: no cover

    @abstractmethod
    def _get_languages(self):
        'AbstractMethod: Returns dict of supported languages and voices'
        pass  # pragma: no cover

    @abstractmethod
    def _say(self, phrase, language, voice, voiceinfo, options):
        '''
        AbstractMethod: Let engin actually says the phrase

        :phrase: The text phrase to say
        :language: The requested language
        :voice: The requested voice
        :voiceinfo: Data about the requested voice
        :options: Extra options
        '''
        pass  # pragma: no cover

    @classmethod
    def get_init_options(cls):
        '''
        Returns a dict describing the engine options.

        Uses cls._get_init_options()
        '''
        options = {
            'enabled': {
                'description': 'Is enabled?',
                'type': 'bool',
                'default': True
            },
        }
        options.update(cls._get_init_options())
        return options

    # Base class continues here
    def __init__(self, **_options):
        self._logger = logging.getLogger(__name__)
        self.ioptions = process_options(self.__class__.get_init_options(), _options, TTSError)

        # Pre-caching potentially slow results
        self.default_language = 'en'
        self.languages_options = {}
        self.default_options = {}
        self.optionspec = None
        self.languages = None
        self.available = self.is_available()
        if self.available:
            self.optionspec = self.get_options()
            self.languages = self.get_languages()
            self.configure_default()

    def is_available(self):
        '''
        Boolean on if engine available.

        Checks if enabled, can output audio and self._is_available()
        '''
        return (
            self.ioptions['enabled']
            and check_executable('aplay')
            and self._is_available()
        )

    def _assert_available(self):
        if not self.ioptions['enabled']:
            raise TTSError('Not enabled')
        if not self.available:
            raise TTSError('Not available')

    def get_options(self):
        '''
        Returns dict of voice options.

        Raises TTSError if not available.
        '''
        self._assert_available()
        return self._get_options()

    def get_languages(self):
        '''
        Returns dict of supported languages and voices.

        Raises TTSError if not available.
        '''
        self._assert_available()
        return self._get_languages()

    def _get_language_options(self, language):
        if language in self.languages_options.keys():
            return self.languages_options[language]
        return None, {}

    def _configure(self, language=None, voice=None, **_options):
        self._assert_available()

        language = language or self.default_language
        lang_voice, lang_options = self._get_language_options(language)
        voice = voice or lang_voice

        if language not in self.languages.keys():
            raise TTSError('Bad language: %s' % language, self.languages.keys())

        voice = voice if voice else self.languages[language]['default']
        if voice not in self.languages[language]['voices'].keys():
            raise TTSError('Bad voice: %s' % voice, self.languages[language]['voices'].keys())
        voiceinfo = self.languages[language]['voices'][voice]

        lang_options.update(_options)
        options = process_options(self.optionspec, lang_options, TTSError)
        return language, voice, voiceinfo, options

    def configure_default(self, **_options):
        '''
        Sets default configuration.

        Raises TTSError on error.
        '''
        language, voice, voiceinfo, options = self._configure(**_options)
        self.languages_options[language] = (voice, options)
        self.default_language = language
        self.default_options = options

    def configure(self, **_options):
        '''
        Sets language-specific configuration.

        Raises TTSError on error.
        '''
        language, voice, voiceinfo, options = self._configure(**_options)
        self.languages_options[language] = (voice, options)

    def say(self, phrase, **_options):
        '''
        Says the phrase, optionally allows to select/override any voice options.
        '''
        language, voice, voiceinfo, options = self._configure(**_options)
        self._logger.debug("Saying '%s' with '%s'", phrase, self.SLUG)
        self._say(phrase, language, voice, voiceinfo, options)

    def play(self, filename, translate=False):  # pragma: no cover
        '''
        Plays the sounds.

        :filename: The input file name
        :translate: If True, it runs it through audioread which will translate from common compression formats to raw WAV.
        '''
        # FIXME: Use platform-independent and async audio-output here
        # PyAudio looks most promising, too bad about:
        #  --allow-external PyAudio --allow-unverified PyAudio
        if translate:
            with tempfile.NamedTemporaryFile(suffix='.wav', delete=False) as f:
                fname = f.name
            with audioread.audio_open(filename) as f:
                with contextlib.closing(wave.open(fname, 'w')) as of:
                    of.setnchannels(f.channels)
                    of.setframerate(f.samplerate)
                    of.setsampwidth(2)
                    for buf in f:
                        of.writeframes(buf)
            filename = fname

        cmd = ['aplay', str(filename)]
        self._logger.debug('Executing %s', ' '.join([pipes.quote(arg) for arg in cmd]))
        subprocess.call(cmd)

        if translate:
            os.remove(fname)



import os
import tempfile
import pipes
from talkey.base import AbstractTTSEngine, subprocess, register
from talkey.utils import check_executable


@register
class FliteTTS(AbstractTTSEngine):
    """
    Uses the flite speech synthesizer.

    Requires ``flite`` to be available.
    """

    SLUG = 'flite'

    @classmethod
    def _get_init_options(cls):
        return {
            'flite': {
                'description': 'FLite executable path',
                'type': 'str',
                'default': 'flite'
            },
        }

    def _is_available(self):
        return check_executable(self.ioptions['flite'])

    def _get_options(self):
        return {}

    def _get_languages(self):
        output = subprocess.check_output(['flite', '-lv'], universal_newlines=True)
        voices = output[output.find(':') + 1:].split()
        return {
            'en': {
                'default': 'kal',
                'voices': dict([(voice, {}) for voice in voices])
            }
        }

    def _say(self, phrase, language, voice, voiceinfo, options):
        cmd = [
            'flite',
            '-voice', voice,
            '-t', phrase
        ]
        with tempfile.NamedTemporaryFile(suffix='.wav', delete=False) as f:
            fname = f.name
        cmd.append(fname)
        self._logger.debug('Executing %s', ' '.join([pipes.quote(arg) for arg in cmd]))
        subprocess.call(cmd)
        self.play(fname)
        os.remove(fname)



from talkey.base import AbstractTTSEngine, DETECTABLE_LANGS


class DummyTTS(AbstractTTSEngine):
    """
    Dummy TTS engine that logs phrases with INFO level instead of synthesizing
    speech.
    """

    SLUG = "dummy"

    @classmethod
    def _get_init_options(cls):
        return {
            'enabled': {
                'description': 'Is enabled?',
                'type': 'bool',
                'default': False,
            },
        }

    def _is_available(self):
        return True

    def _get_options(self):
        return {}

    def _get_languages(self):
        return dict([
            (lang, {'default': lang, 'voices': {lang: {}}})
            for lang in DETECTABLE_LANGS
        ])

    def _say(self, phrase, language, voice, voiceinfo, options):
        self._logger.info('%s: %s' % (language, phrase))



'''
talkey test suite
'''
# pylint: disable=W0104
from talkey.base import DETECTABLE_LANGS, TTSError, AbstractTTSEngine, subprocess
from talkey.engines import *
from talkey.utils import check_executable, process_options
from talkey.tts import create_engine, Talkey

from os.path import isfile

try:
    import unittest2 as unittest  # pylint: disable=F0401
except ImportError:
    import unittest

LAST_PLAY = None


# Patch AbstractTTSEngine.play to record its params
# Really need to make this loosely connected
def fakeplay(self, filename, translate=False):
    global LAST_PLAY
    output = subprocess.check_output(['file', filename], universal_newlines=True)
    LAST_PLAY = (self, filename, output)
AbstractTTSEngine.play = fakeplay


class CheckExecutableTest(unittest.TestCase):

    def test_check_executable_found(self):
        self.assertTrue(check_executable('file'))

    def test_check_executable_not_found(self):
        self.assertFalse(check_executable('aieaoauu_not-findable_lfsdauybqwer'))


class ProcessOptionsTest(unittest.TestCase):

    def test_process_options_unknown_param(self):
        with self.assertRaisesRegexp(TTSError, 'Unknown options'):
            process_options({}, {'test': 'fail'}, TTSError)

    def test_process_options_bad_type(self):
        with self.assertRaisesRegexp(TTSError, 'Bad type: garbage'):
            process_options({'test': {'type': 'garbage'}}, {'test': 'fail'}, TTSError)

    def test_process_options_default_none(self):
        spec = {'test': {'type': 'str'}}
        ret = process_options(spec, {}, TTSError)
        self.assertEqual(ret, {'test': None})

    def test_process_options_default_provided(self):
        spec = {'test': {'type': 'str', 'default': 'moo'}}
        ret = process_options(spec, {}, TTSError)
        self.assertEqual(ret, {'test': 'moo'})

    def test_process_options_bool(self):
        spec = {'test': {'type': 'bool'}}
        ret = process_options(spec, {}, TTSError)
        self.assertEqual(ret, {'test': False})
        ret = process_options(spec, {'test': 'yEs'}, TTSError)
        self.assertEqual(ret, {'test': True})

    def test_process_options_int(self):
        spec = {'test': {'type': 'int', 'min': 5, 'max': 8}}
        with self.assertRaises(TypeError):
            process_options(spec, {}, TTSError)
        ret = process_options(spec, {'test': '6'}, TTSError)
        self.assertEqual(ret, {'test': 6})
        ret = process_options(spec, {'test': '6.2'}, TTSError)
        self.assertEqual(ret, {'test': 6})
        ret = process_options(spec, {'test': 6}, TTSError)
        self.assertEqual(ret, {'test': 6})
        ret = process_options(spec, {'test': 6.1}, TTSError)
        self.assertEqual(ret, {'test': 6})

    def test_process_options_int_min(self):
        with self.assertRaisesRegexp(TTSError, 'Min is 5'):
            process_options({'test': {'type': 'int', 'min': 5}}, {'test': 3}, TTSError)
        process_options({'test': {'type': 'int'}}, {'test': 3}, TTSError)

    def test_process_options_int_max(self):
        with self.assertRaisesRegexp(TTSError, 'Max is 8'):
            process_options({'test': {'type': 'int', 'max': 8}}, {'test': 10}, TTSError)
        process_options({'test': {'type': 'int'}}, {'test': 10}, TTSError)

    def test_process_options_float(self):
        spec = {'test': {'type': 'float', 'min': 5.2, 'max': 8.9}}
        with self.assertRaises(TypeError):
            process_options(spec, {}, TTSError)
        ret = process_options(spec, {'test': '6'}, TTSError)
        self.assertEqual(ret, {'test': 6.0})
        ret = process_options(spec, {'test': '6.2'}, TTSError)
        self.assertEqual(ret, {'test': 6.2})
        ret = process_options(spec, {'test': 6}, TTSError)
        self.assertEqual(ret, {'test': 6.0})
        ret = process_options(spec, {'test': 6.1}, TTSError)
        self.assertEqual(ret, {'test': 6.1})

    def test_process_options_float_min(self):
        with self.assertRaisesRegexp(TTSError, 'Min is 5.2'):
            process_options({'test': {'type': 'float', 'min': 5.2}}, {'test': 3}, TTSError)
        process_options({'test': {'type': 'float'}}, {'test': 3}, TTSError)

    def test_process_options_float_max(self):
        with self.assertRaisesRegexp(TTSError, 'Max is 8.9'):
            process_options({'test': {'type': 'float', 'max': 8.9}}, {'test': 10}, TTSError)
        process_options({'test': {'type': 'float'}}, {'test': 10}, TTSError)

    def test_process_options_enum(self):
        spec = {'test': {'type': 'enum', 'values': ['one', 'two', 'four']}}
        with self.assertRaisesRegexp(TTSError, 'Bad test value: None'):
            process_options(spec, {}, TTSError)
        with self.assertRaisesRegexp(TTSError, 'Bad test value: three'):
            process_options(spec, {'test': 'three'}, TTSError)
        ret = process_options(spec, {'test': 'two'}, TTSError)
        self.assertEqual(ret, {'test': 'two'})


class CreateEngineTest(unittest.TestCase):

    def test_create_engine(self):
        eng = create_engine('dummy', options={'enabled': True})
        self.assertEqual(eng.SLUG, 'dummy')
        self.assertTrue(eng.available)
        assert eng.languages

    def test_create_engine_bad(self):
        with self.assertRaisesRegexp(TTSError, 'Unknown engine'):
            create_engine('baddy')


class TalkeyTest(unittest.TestCase):
    TXTS = [
        # Actual, Unweighted, Text
        ('en', 'pl', 'Cows go moo'),
        ('en', 'en', 'Old McDonald had a farm'),
        ('af', 'nl', "Ou boer McDonald het 'n plaas gehad"),
    ]

    def setUp(self):
        global LAST_PLAY
        LAST_PLAY = None

    def test_create_basic(self):
        tts = Talkey()
        for txt in self.TXTS:
            self.assertEqual(txt[1], tts.classify(txt[2]))

        self.assertEqual(tts.get_engine_for_lang('en').SLUG, 'espeak')

        tts.say('Old McDonald had a farm')
        inst, filename, output = LAST_PLAY
        self.assertIn('WAVE audio', output)
        self.assertEqual(inst, tts.engines[0])
        self.assertFalse(isfile(filename), 'Tempfile not deleted')

    def test_create_weighted(self):
        tts = Talkey(preferred_languages=['en', 'af'])
        for txt in self.TXTS:
            self.assertEqual(txt[0], tts.classify(txt[2]))

        tts = Talkey(preferred_languages=['en', 'af'], preferred_factor=10.0)
        self.assertEqual(self.TXTS[2][1], tts.classify(self.TXTS[2][2]))

    def test_create_empty(self):
        with self.assertRaisesRegexp(TTSError, 'No supported languages'):
            Talkey(
                espeak={'options': {'enabled': False}},
                festival={'options': {'enabled': False}},
                pico={'options': {'enabled': False}},
                flite={'options': {'enabled': False}},
            )

    def test_create_language_config(self):
        tts = Talkey(
            espeak={
                'languages': {
                    'en': {
                        'voice': 'english-mb-en1',
                        'words_per_minute': 130
                    },
                }
            },
        )
        eng = tts.engines[0]
        self.assertEqual(eng.languages_options['en'][0], 'english-mb-en1')
        self.assertEqual(eng.languages_options['en'][1]['words_per_minute'], 130)

    def test_get_engine_for_lang(self):
        tts = Talkey(
            espeak={'options': {'enabled': False}},
        )
        self.assertEqual(tts.get_engine_for_lang('fr').SLUG, 'pico')
        with self.assertRaisesRegexp(TTSError, 'Could not match language'):
            tts.get_engine_for_lang('af')

    def test_engine_preference(self):
        tts = Talkey(engine_preference=['pico'])

        self.assertEqual(tts.engines[0].SLUG, 'pico')
        self.assertEqual(tts.engines[1].SLUG, 'espeak')


class BaseTTSTest(unittest.TestCase):
    '''
    Tests talkey basic functionality
    '''
    # pylint: disable=E1102
    CLS = None
    SLUG = None
    INIT_ATTRS = ['enabled']
    CONF = {}
    OBJ_ATTRS = []
    EVAL_PLAY = True
    SKIP_IF_NOT_AVAILABLE = False
    FILE_TYPE = 'WAVE audio'

    @classmethod
    def setUpClass(cls):
        if not cls.CLS:
            raise unittest.SkipTest()

    def setUp(self):
        global LAST_PLAY
        LAST_PLAY = None

        # Stupid py2.6, Grrr
        if not self.CLS:
            raise unittest.SkipTest()

    def skip_not_available(self):
        if self.SKIP_IF_NOT_AVAILABLE and not self.CLS(**self.CONF).is_available():
            # Skip networked engines if not available to prevent spurious failiures
            raise unittest.SkipTest()  # pragma: no cover

    def test_class_init_options(self):
        cls = self.CLS
        self.assertEqual(cls.SLUG, self.SLUG)
        self.assertEqual(sorted(cls.get_init_options().keys()), sorted(self.INIT_ATTRS))

    def test_configure_not_enabled(self):
        with self.assertRaisesRegexp(TTSError, 'Not enabled'):
            self.CLS(enabled=False).configure()

    def test_class_instantiation(self):
        self.skip_not_available()
        obj = self.CLS(**self.CONF)
        self.assertEqual(obj.SLUG, self.SLUG)
        self.assertEqual(obj.is_available(), True)
        self.assertEqual(sorted(obj.get_options().keys()), sorted(self.OBJ_ATTRS))

    def test_class_configure(self):
        self.skip_not_available()
        obj = self.CLS(**self.CONF)
        language, voice, voiceinfo, options = obj._configure()
        self.assertEqual(language, 'en')
        self.assertIsNotNone(voice)
        self.assertEqual(voice, obj.get_languages()['en']['default'])
        self.assertEqual(sorted(options.keys()), sorted(self.OBJ_ATTRS))

    def test_class_say(self):
        self.skip_not_available()
        obj = self.CLS(**self.CONF)
        obj.say('Cows go moo')
        if self.EVAL_PLAY:
            inst, filename, output = LAST_PLAY
            self.assertIn(self.FILE_TYPE, output)
            self.assertEqual(inst, obj)
            self.assertFalse(isfile(filename), 'Tempfile not deleted')


class DummyTTSTest(BaseTTSTest):
    CLS = DummyTTS
    SLUG = 'dummy'
    CONF = {'enabled': True}
    EVAL_PLAY = False

    def test_configure_bad_language(self):
        with self.assertRaisesRegexp(TTSError, 'Bad language'):
            self.CLS(enabled=True).configure(language='bad')

    def test_configure_bad_voice(self):
        with self.assertRaisesRegexp(TTSError, 'Bad voice'):
            self.CLS(enabled=True).configure(voice='bad')


class FestivalTTSTest(BaseTTSTest):
    CLS = FestivalTTS
    SLUG = 'festival'
    INIT_ATTRS = ['enabled', 'festival']


class FliteTTSTest(BaseTTSTest):
    CLS = FliteTTS
    SLUG = 'flite'
    INIT_ATTRS = ['enabled', 'flite']


class EspeakTTSTest(BaseTTSTest):
    CLS = EspeakTTS
    SLUG = 'espeak'
    INIT_ATTRS = ['enabled', 'espeak', 'mbrola', 'mbrola_voices', 'passable_only']
    OBJ_ATTRS = ['words_per_minute', 'pitch_adjustment', 'variant']
    EVAL_PLAY = True

    def test_get_languages_options(self):
        pat = self.CLS(passable_only=True).get_languages()
        paf = self.CLS(passable_only=False).get_languages()
        assert len(paf.keys()) > len(pat.keys())

    def test_mbrola_language(self):
        obj = self.CLS(**self.CONF)
        obj.say('Cows go moo', voice='english-mb-en1')
        inst, filename, output = LAST_PLAY
        self.assertIn(self.FILE_TYPE, output)
        self.assertEqual(inst, obj)
        self.assertFalse(isfile(filename), 'Tempfile not deleted')

    def test_enabled_not_available(self):
        with self.assertRaisesRegexp(TTSError, 'Not available'):
            self.CLS(enabled=True, espeak='badexec').configure()

    def test_no_mbrola(self):
        obj = self.CLS(enabled=True)
        assert 'english-mb-en1' in obj.languages['en']['voices'].keys()

        obj = self.CLS(enabled=True, mbrola='badexec')
        assert 'english-mb-en1' not in obj.languages['en']['voices'].keys()


class PicoTTSTest(BaseTTSTest):
    CLS = PicoTTS
    SLUG = 'pico'
    INIT_ATTRS = ['enabled', 'pico2wave']


class MaryTTSTest(BaseTTSTest):
    CLS = MaryTTS
    SLUG = 'mary'
    INIT_ATTRS = ['enabled', 'host', 'port', 'scheme']
    CONF = {'enabled': True}#, 'host': 'mary.dfki.de'}
    EVAL_PLAY = True
    SKIP_IF_NOT_AVAILABLE = True


class GoogleTTSTest(BaseTTSTest):
    CLS = GoogleTTS
    SLUG = 'google'
    CONF = {'enabled': True}
    FILE_TYPE = 'MPEG ADTS, layer III'
    SKIP_IF_NOT_AVAILABLE = True



import os
import tempfile
import pipes
from talkey.base import AbstractTTSEngine, subprocess, register
from talkey.utils import check_executable


@register
class EspeakTTS(AbstractTTSEngine):
    """
    Uses the eSpeak speech synthesizer.

    Requires ``espeak`` and optionally ``mbrola`` to be available.
    """

    SLUG = "espeak"
    # http://espeak.sourceforge.net/languages.html
    QUALITY_LANGS = [
        'en', 'af', 'bs', 'ca', 'cs', 'da', 'de', 'el', 'eo', 'es',
        'fi', 'fr', 'hr', 'hu', 'it', 'kn', 'ku', 'lv', 'nl', 'pl',
        'pt', 'ro', 'sk', 'sr', 'sv', 'sw', 'ta', 'tr', 'zh'
    ]

    @classmethod
    def _get_init_options(cls):
        return {
            'espeak': {
                'description': 'eSpeak executable path',
                'type': 'str',
                'default': 'espeak'
            },
            'mbrola': {
                'description': 'mbrola executable path',
                'type': 'str',
                'default': 'mbrola'
            },
            'mbrola_voices': {
                'description': 'mbrola voices path',
                'type': 'str',
                'default': '/usr/share/mbrola'
            },
            'passable_only': {
                'description': 'Only allow languages of passable quality, as per http://espeak.sourceforge.net/languages.html',
                'type': 'bool',
                'default': True
            }
        }

    def _is_available(self):
        return check_executable(self.ioptions['espeak'])

    def has_mbrola(self):
        return check_executable(self.ioptions['mbrola'])

    def _get_options(self):
        output = subprocess.check_output([self.ioptions['espeak'], '--voices=variant'], universal_newlines=True)
        variants = [row[row.find('!v/') + 3:].strip() for row in output.split('\n')[1:] if row]
        return {
            'variant': {
                'type': 'enum',
                'values': sorted([''] + variants),
                'default': 'm3',
            },
            'pitch_adjustment': {
                'type': 'int',
                'min': 0,
                'max': 99,
                'default': 50,
            },
            'words_per_minute': {
                'type': 'int',
                'min': 80,
                'max': 450,
                'default': 150,
            },
        }

    def _get_languages(self):
        'Get all working voices and languages for eSpeak'

        def fix_voice(voice):
            'Work-around bug in eSpeak 1.46 where gender is sometimes missing'
            if voice[2] not in ['M', 'F', '-']:
                return voice[:2] + ['-'] + voice[2:]  # pragma: no cover
            return voice

        output = subprocess.check_output([self.ioptions['espeak'], '--voices'], universal_newlines=True)
        voices = [['espeak'] + fix_voice(row.split())[:4] for row in output.split('\n')[1:] if row]

        if self.has_mbrola():
            output = subprocess.check_output([self.ioptions['espeak'], '--voices=mbrola'], universal_newlines=True)
            mvoices = [fix_voice(row.split())[:5] for row in output.split('\n')[1:] if row]
            for mvoice in mvoices:
                mbfile = mvoice[4].split('-')[1]
                mbfile = os.path.join(self.ioptions['mbrola_voices'], mbfile, mbfile)
                if os.path.isfile(mbfile):
                    voices.append(['mbrola'] + mvoice)

        langs = set([voice[2].split('-')[0] for voice in voices])
        if self.ioptions['passable_only']:
            langs = [lang for lang in langs if lang in self.QUALITY_LANGS]
        tree = dict([(lang, {'voices': {}}) for lang in langs])
        for voice in voices:
            lang = voice[2].split('-')[0]
            if lang in langs:
                tree[lang]['voices'][voice[4]] = {'gender': voice[3], 'pty': int(voice[1]), 'type': voice[0]}
        for lang in langs:
            # Try to find sane default voice, score by pty, then take shortest (for determinism)
            vcs = tree[lang]['voices']
            pty = min([v['pty'] for v in vcs.values()])
            tree[lang]['default'] = sorted([k for k, v in vcs.items() if v['pty'] == pty])[0]
        return tree

    def _say(self, phrase, language, voice, voiceinfo, options):
        with tempfile.NamedTemporaryFile(suffix='.wav', delete=False) as f:
            fname = f.name
        vce = voice
        if voiceinfo['type'] == 'espeak' and options['variant']:
            vce += '+' + options['variant']
        cmd = [
            self.ioptions['espeak'],
            '-v', vce,
            '-p', options['pitch_adjustment'],
            '-s', options['words_per_minute'],
            '-w', fname,
            phrase
        ]
        cmd = [str(x) for x in cmd]
        self._logger.debug('Executing %s', ' '.join([pipes.quote(arg) for arg in cmd]))
        subprocess.call(cmd)
        self.play(fname)
        os.remove(fname)



import os
import tempfile
import requests

try:
    # pylint: disable=E0611
    from urlparse import urlunsplit
    from urllib import urlencode
except ImportError:
    # pylint: disable=E0611
    from urllib.parse import urlunsplit, urlencode

from talkey.base import AbstractTTSEngine, register
from talkey.utils import check_network_connection


@register
class MaryTTS(AbstractTTSEngine):
    """
    Uses the MARY Text-to-Speech System (MaryTTS)
    MaryTTS is an open-source, multilingual Text-to-Speech Synthesis platform
    written in Java.
    Please specify your own server instead of using the demonstration server
    (http://mary.dfki.de:59125/) to save bandwidth and to protect your privacy.
    """

    SLUG = "mary"

    @classmethod
    def _get_init_options(cls):
        return {
            'enabled': {
                'description': 'Is enabled?',
                'type': 'bool',
                'default': False,
            },
            'scheme': {
                'description': 'HTTP schema',
                'type': 'enum',
                'default': 'http',
                'values': ['http', 'https'],
            },
            'host': {
                'description': 'Mary server address',
                'type': 'str',
                'default': '127.0.0.1',
            },
            'port': {
                'description': 'Mary server port',
                'type': 'int',
                'default': 59125,
                'min': 1,
                'max': 65535,
            }
        }

    def _makeurl(self, path, query={}):
        query_s = urlencode(query)
        urlparts = (self.ioptions['scheme'], self.ioptions['host'] + ':' + str(self.ioptions['port']), path, query_s, '')
        return urlunsplit(urlparts)

    def _is_available(self):
        return check_network_connection(self.ioptions['host'], self.ioptions['port'])

    def _get_options(self):
        return {}

    def _get_languages(self):
        res = requests.get(self._makeurl('voices')).text
        langs = {}
        for voice in [row.split() for row in res.split('\n') if row]:
            lang = voice[1].split('_')[0]
            langs.setdefault(lang, {'default': voice[0], 'voices': {}})
            langs[lang]['voices'][voice[0]] = {
                'gender': voice[2],
                'locale': voice[1]
            }
        return langs

    def _say(self, phrase, language, voice, voiceinfo, options):
        query = {'OUTPUT_TYPE': 'AUDIO',
                 'AUDIO': 'WAVE_FILE',
                 'INPUT_TYPE': 'TEXT',
                 'INPUT_TEXT': phrase,
                 'LOCALE': voiceinfo['locale'],
                 'VOICE': voice}

        res = requests.get(self._makeurl('/process', query=query))
        with tempfile.NamedTemporaryFile(suffix='.wav', delete=False) as f:
            f.write(res.content)
            tmpfile = f.name
        self.play(tmpfile)
        os.remove(tmpfile)



import sys
from setuptools import setup, find_packages

def get_version(fname):
    import re
    verstrline = open(fname, "rt").read()
    mo = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", verstrline, re.M)
    if mo:
        return mo.group(1)
    else:
        raise RuntimeError("Unable to find version string in %s." % (fname,))

def get_requirements():
    requirements = [
        'langid',
        'requests',
        'audioread',
    ]
    if sys.version_info[0] == 2:
        requirements.append('subprocess32')
    return requirements

def get_test_requirements():
    requirements = [
        'gtts',
    ]
    if sys.version_info[0:2] == (2, 6):
        requirements.append('unittest2')
    return requirements

setup(
    name='talkey',
    version=get_version('talkey/__init__.py'),
    description='Simple Test-To-Speech (TTS) interface library with multi-language and multi-engine support.',
    long_description=open('README.rst').read(),
    author='Nickolas Grigoriadis',
    author_email='nagrigoriadis@gmail.com',
    url='https://github.com/grigi/talkey',
    zip_safe=False,
    test_suite='talkey.test_suite',

    # Dependencies
    install_requires=get_requirements(),
    tests_require=get_test_requirements(),

    # Packages
    packages=find_packages(),
    include_package_data=True,

    # Scripts
    scripts=[],

    # Classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Multimedia :: Sound/Audio :: Speech',
    ]
)




import os
import tempfile
import pipes
from talkey.base import AbstractTTSEngine, subprocess, register
from talkey.utils import check_executable


@register
class FestivalTTS(AbstractTTSEngine):
    """
    Uses the festival speech synthesizer.

    Requires ``festival`` to be available.
    """

    SLUG = 'festival'

    SAY_TEMPLATE = """(Parameter.set 'Audio_Required_Format 'riff)
(Parameter.set 'Audio_Command "mv $FILE {outfilename}")
(Parameter.set 'Audio_Method 'Audio_Command)
(SayText "{phrase}")
"""

    @classmethod
    def _get_init_options(cls):
        return {
            'festival': {
                'description': 'Festival executable path',
                'type': 'str',
                'default': 'festival'
            },
        }

    def _is_available(self):
        if check_executable(self.ioptions['festival']):
            cmd = ['festival', '--pipe']
            with tempfile.SpooledTemporaryFile() as in_f:
                self._logger.debug('Executing %s', ' '.join([pipes.quote(arg) for arg in cmd]))
                output = subprocess.check_output(cmd, stdin=in_f, stderr=subprocess.STDOUT, universal_newlines=True).strip()
                return 'No default voice found' not in output
        return False  # pragma: no cover

    def _get_options(self):
        return {}

    def _get_languages(self):
        return {
            'en': {'default': 'en', 'voices': {'en': {}}}
        }

    def _say(self, phrase, language, voice, voiceinfo, options):
        cmd = ['festival', '--pipe']
        with tempfile.NamedTemporaryFile(suffix='.wav', delete=False) as f:
            fname = f.name
        with tempfile.SpooledTemporaryFile() as in_f:
            in_f.write(self.SAY_TEMPLATE.format(outfilename=fname, phrase=phrase.replace('\\', '\\\\"').replace('"', '\\"')).encode('utf-8'))
            in_f.seek(0)
            self._logger.debug('Executing %s', ' '.join([pipes.quote(arg) for arg in cmd]))
            subprocess.call(cmd, stdin=in_f)
        self.play(fname)
        os.remove(fname)
